@@ -106,17 +106,10 @@ def check_policy(context, action, target):
     nova.policy.enforce(context, _action, target)
 
 
-class API(base.Base):
-    """API for interacting with the compute manager."""
-
-    def __init__(self, image_service=None, network_api=None, volume_api=None,
-                 **kwargs):
-        self.image_service = (image_service or
-                              nova.image.get_default_image_service())
-
-        self.network_api = network_api or network.API()
-        self.volume_api = volume_api or volume.API()
-        super(API, self).__init__(**kwargs)
+class BaseAPI(base.Base):
+    """Base API class."""
+    def __init__(self, **kwargs):
+        super(BaseAPI, self).__init__(**kwargs)
 
     def _cast_or_call_compute_message(self, rpc_method, compute_method,
             context, instance=None, host=None, params=None):
@@ -157,10 +150,25 @@ class API(base.Base):
         """Generic handler for RPC calls to compute."""
         return self._cast_or_call_compute_message(rpc.call, *args, **kwargs)
 
-    def _cast_scheduler_message(self, context, args):
+    @staticmethod
+    def _cast_scheduler_message(context, args):
         """Generic handler for RPC calls to the scheduler."""
         rpc.cast(context, FLAGS.scheduler_topic, args)
 
+
+class API(BaseAPI):
+    """API for interacting with the compute manager."""
+
+    def __init__(self, image_service=None, network_api=None, volume_api=None,
+                 **kwargs):
+        self.image_service = (image_service or
+                              nova.image.get_default_image_service())
+
+        self.network_api = network_api or network.API()
+        self.volume_api = volume_api or volume.API()
+        self.sgh = utils.import_object(FLAGS.security_group_handler)
+        super(API, self).__init__(**kwargs)
+
     def _check_injected_file_quota(self, context, injected_files):
         """Enforce quota limits on injected files.
 
@@ -701,6 +709,7 @@ class API(base.Base):
                       'user_id': context.user_id,
                       'project_id': context.project_id}
             self.db.security_group_create(context, values)
+            self.sgh.trigger_security_group_create_refresh(context, values)
 
     def trigger_security_group_rules_refresh(self, context, security_group_id):
         """Called when a rule is added to or removed from a security_group."""
@@ -1632,6 +1641,8 @@ class API(base.Base):
 
         self.network_api.associate_floating_ip(context,
                 floating_address=address, fixed_address=ips[0]['address'])
+        self.network_api.invalidate_instance_cache(context.elevated(),
+                                                   instance)
 
     @wrap_check_policy
     def get_instance_metadata(self, context, instance):
@@ -1677,11 +1688,8 @@ class API(base.Base):
         return self.db.instance_fault_get_by_instance_uuids(context, uuids)
 
 
-class HostAPI(base.Base):
+class HostAPI(BaseAPI):
     """Sub-set of the Compute Manager API for managing host operations."""
-    def __init__(self, **kwargs):
-        super(HostAPI, self).__init__(**kwargs)
-
     def set_host_enabled(self, context, host, enabled):
         """Sets the specified host's ability to accept new instances."""
         # NOTE(comstud): No instance_uuid argument to this compute manager
