@@ -627,6 +627,7 @@ def get_dhcp_opts(context, network_ref):
                 default_gw_vif[instance_id] = vifs[0]['id']
 
         for datum in data:
+            instance_id = datum['instance_id']
             if instance_id in default_gw_vif:
                 # we don't want default gateway for this fixed ip
                 if default_gw_vif[instance_id] != datum['vif_id']:
@@ -689,6 +690,7 @@ def restart_dhcp(context, dev, network_ref):
         if conffile.split("/")[-1] in out:
             try:
                 _execute('kill', '-HUP', pid, run_as_root=True)
+                _add_dnsmasq_accept_rules(dev)
                 return
             except Exception as exc:  # pylint: disable=W0703
                 LOG.error(_('Hupping dnsmasq threw %s'), exc)
